"""A `SceneEditor` for the `SceneModel`.
"""

# Authors: Prabhu Ramachandran <prabhu [at] aero.iitb.ac.in>
#          Robert Kern <robert.kern [at] gmail.com>
#
# Copyright (c) 2007, Enthought, Inc.
# License: BSD Style.

# Import the toolkit specific version.
from tvtk.pyface.toolkit import toolkit_object
SceneEditor = toolkit_object('scene_editor:SceneEditor')
